# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .. import core
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, BaseHttpResponse, SyncPager
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..types.video_indexing_task import VideoIndexingTask
from .types.tasks_create_response import TasksCreateResponse
from .types.tasks_list_request_status_item import TasksListRequestStatusItem
from .types.tasks_list_response import TasksListResponse
from .types.tasks_retrieve_response import TasksRetrieveResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTasksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        index_id: typing.Optional[str] = None,
        status: typing.Optional[
            typing.Union[TasksListRequestStatusItem, typing.Sequence[TasksListRequestStatusItem]]
        ] = None,
        filename: typing.Optional[str] = None,
        duration: typing.Optional[float] = None,
        width: typing.Optional[int] = None,
        height: typing.Optional[int] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[VideoIndexingTask]:
        """
        This method returns a list of the video indexing tasks in your account. The platform returns your video indexing tasks sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        index_id : typing.Optional[str]
            Filter by the unique identifier of an index.

        status : typing.Optional[typing.Union[TasksListRequestStatusItem, typing.Sequence[TasksListRequestStatusItem]]]
            Filter by one or more video indexing task statuses. The following options are available:
            - `ready`: The video has been successfully uploaded and indexed.
            - `uploading`: The video is being uploaded.
            - `validating`: The video is being validated against the prerequisites.
            - `pending`: The video is pending.
            - `queued`: The video is queued.
            - `indexing`: The video is being indexed.
            - `failed`: The video indexing task failed.

            To filter by multiple statuses, specify the `status` parameter for each value:
            ```
            status=ready&status=validating
            ```

        filename : typing.Optional[str]
            Filter by filename.

        duration : typing.Optional[float]
            Filter by duration. Expressed in seconds.

        width : typing.Optional[int]
            Filter by width.

        height : typing.Optional[int]
            Filter by height.

        created_at : typing.Optional[str]
            Filter video indexing tasks by the creation date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            Filter video indexing tasks by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were updated on the specified date at or after the given time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[VideoIndexingTask]
            The video indexing tasks have successfully been retrieved.
        """
        page = page if page is not None else 1

        _response = self._client_wrapper.httpx_client.request(
            "tasks",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
                "sort_by": sort_by,
                "sort_option": sort_option,
                "index_id": index_id,
                "status": status,
                "filename": filename,
                "duration": duration,
                "width": width,
                "height": height,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    TasksListResponse,
                    parse_obj_as(
                        type_=TasksListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True
                _get_next = lambda: self.list(
                    page=page + 1,
                    page_limit=page_limit,
                    sort_by=sort_by,
                    sort_option=sort_option,
                    index_id=index_id,
                    status=status,
                    filename=filename,
                    duration=duration,
                    width=width,
                    height=height,
                    created_at=created_at,
                    updated_at=updated_at,
                    request_options=request_options,
                )
                return SyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        index_id: str,
        video_file: typing.Optional[core.File] = OMIT,
        video_url: typing.Optional[str] = OMIT,
        enable_video_stream: typing.Optional[bool] = OMIT,
        user_metadata: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TasksCreateResponse]:
        """
        This method creates a video indexing task that uploads and indexes a video in a single operation.

        <Warning title="Legacy endpoint">
        This endpoint bundles two operations (upload and indexing) together. In the next major API release, this endpoint will be removed in favor of a separated workflow:
        1. Upload your video using the [`POST /assets`](/v1.3/api-reference/upload-content/direct-uploads/create) endpoint
        2. Index the uploaded video using the [`POST /indexes/{index-id}/indexed-assets`](/v1.3/api-reference/index-content/create) endpoint

        This separation provides better control, reusability of assets, and improved error handling. New implementations should use the new workflow.
        </Warning>


        Upload options:
        - **Local file**: Use the `video_file` parameter.
        - **Publicly accessible URL**: Use the `video_url` parameter.

        Your video files must meet requirements based on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements).
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#video-file-requirements).
        - If you want to both search and analyze your videos, the most restrictive requirements apply.
        - This method allows you to upload files up to 2 GB in size. To upload larger files, use the [Multipart Upload API](/v1.3/api-reference/upload-content/multipart-uploads)

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video is being uploaded.

        video_file : typing.Optional[core.File]
            See core.File for more documentation

        video_url : typing.Optional[str]
            Specify this parameter to upload a video from a publicly accessible URL.

        enable_video_stream : typing.Optional[bool]
            This parameter indicates if the platform stores the video for streaming. When set to `true`, the platform stores the video, and you can retrieve its URL by calling the [`GET`](/v1.3/api-reference/videos/retrieve) method of the `/indexes/{index-id}/videos/{video-id}` endpoint. You can then use this URL to access the stream over the <a href="https://en.wikipedia.org/wiki/HTTP_Live_Streaming" target="_blank">HLS</a> protocol.

        user_metadata : typing.Optional[str]
            Metadata that helps you categorize your videos. You can specify a list of keys and values. Keys must be of type `string`, and values can be of the following types: `string`, `integer`, `float` or `boolean`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TasksCreateResponse]
            A video indexing task has successfully been created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "tasks",
            method="POST",
            data={
                "index_id": index_id,
                "video_url": video_url,
                "enable_video_stream": enable_video_stream,
                "user_metadata": user_metadata,
            },
            files={
                **({"video_file": video_file} if video_file is not None else {}),
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TasksCreateResponse,
                    parse_obj_as(
                        type_=TasksCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def retrieve(
        self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TasksRetrieveResponse]:
        """
        This method retrieves a video indexing task.

        Parameters
        ----------
        task_id : str
            The unique identifier of the video indexing task to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TasksRetrieveResponse]
            The specified video indexing task has successfully been retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tasks/{jsonable_encoder(task_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TasksRetrieveResponse,
                    parse_obj_as(
                        type_=TasksRetrieveResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        This action cannot be undone.
        Note the following about deleting a video indexing task:
        - You can only delete video indexing tasks for which the status is `ready` or `failed`.
        - If the status of your video indexing task is `ready`, you must first delete the video vector associated with your video indexing task by calling the [`DELETE`](/v1.3/api-reference/videos/delete) method of the `/indexes/videos` endpoint.

        Parameters
        ----------
        task_id : str
            The unique identifier of the video indexing task you want to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tasks/{jsonable_encoder(task_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTasksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        index_id: typing.Optional[str] = None,
        status: typing.Optional[
            typing.Union[TasksListRequestStatusItem, typing.Sequence[TasksListRequestStatusItem]]
        ] = None,
        filename: typing.Optional[str] = None,
        duration: typing.Optional[float] = None,
        width: typing.Optional[int] = None,
        height: typing.Optional[int] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[VideoIndexingTask]:
        """
        This method returns a list of the video indexing tasks in your account. The platform returns your video indexing tasks sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        index_id : typing.Optional[str]
            Filter by the unique identifier of an index.

        status : typing.Optional[typing.Union[TasksListRequestStatusItem, typing.Sequence[TasksListRequestStatusItem]]]
            Filter by one or more video indexing task statuses. The following options are available:
            - `ready`: The video has been successfully uploaded and indexed.
            - `uploading`: The video is being uploaded.
            - `validating`: The video is being validated against the prerequisites.
            - `pending`: The video is pending.
            - `queued`: The video is queued.
            - `indexing`: The video is being indexed.
            - `failed`: The video indexing task failed.

            To filter by multiple statuses, specify the `status` parameter for each value:
            ```
            status=ready&status=validating
            ```

        filename : typing.Optional[str]
            Filter by filename.

        duration : typing.Optional[float]
            Filter by duration. Expressed in seconds.

        width : typing.Optional[int]
            Filter by width.

        height : typing.Optional[int]
            Filter by height.

        created_at : typing.Optional[str]
            Filter video indexing tasks by the creation date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            Filter video indexing tasks by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were updated on the specified date at or after the given time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[VideoIndexingTask]
            The video indexing tasks have successfully been retrieved.
        """
        page = page if page is not None else 1

        _response = await self._client_wrapper.httpx_client.request(
            "tasks",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
                "sort_by": sort_by,
                "sort_option": sort_option,
                "index_id": index_id,
                "status": status,
                "filename": filename,
                "duration": duration,
                "width": width,
                "height": height,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    TasksListResponse,
                    parse_obj_as(
                        type_=TasksListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True

                async def _get_next():
                    return await self.list(
                        page=page + 1,
                        page_limit=page_limit,
                        sort_by=sort_by,
                        sort_option=sort_option,
                        index_id=index_id,
                        status=status,
                        filename=filename,
                        duration=duration,
                        width=width,
                        height=height,
                        created_at=created_at,
                        updated_at=updated_at,
                        request_options=request_options,
                    )

                return AsyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        index_id: str,
        video_file: typing.Optional[core.File] = OMIT,
        video_url: typing.Optional[str] = OMIT,
        enable_video_stream: typing.Optional[bool] = OMIT,
        user_metadata: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TasksCreateResponse]:
        """
        This method creates a video indexing task that uploads and indexes a video in a single operation.

        <Warning title="Legacy endpoint">
        This endpoint bundles two operations (upload and indexing) together. In the next major API release, this endpoint will be removed in favor of a separated workflow:
        1. Upload your video using the [`POST /assets`](/v1.3/api-reference/upload-content/direct-uploads/create) endpoint
        2. Index the uploaded video using the [`POST /indexes/{index-id}/indexed-assets`](/v1.3/api-reference/index-content/create) endpoint

        This separation provides better control, reusability of assets, and improved error handling. New implementations should use the new workflow.
        </Warning>


        Upload options:
        - **Local file**: Use the `video_file` parameter.
        - **Publicly accessible URL**: Use the `video_url` parameter.

        Your video files must meet requirements based on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements).
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#video-file-requirements).
        - If you want to both search and analyze your videos, the most restrictive requirements apply.
        - This method allows you to upload files up to 2 GB in size. To upload larger files, use the [Multipart Upload API](/v1.3/api-reference/upload-content/multipart-uploads)

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video is being uploaded.

        video_file : typing.Optional[core.File]
            See core.File for more documentation

        video_url : typing.Optional[str]
            Specify this parameter to upload a video from a publicly accessible URL.

        enable_video_stream : typing.Optional[bool]
            This parameter indicates if the platform stores the video for streaming. When set to `true`, the platform stores the video, and you can retrieve its URL by calling the [`GET`](/v1.3/api-reference/videos/retrieve) method of the `/indexes/{index-id}/videos/{video-id}` endpoint. You can then use this URL to access the stream over the <a href="https://en.wikipedia.org/wiki/HTTP_Live_Streaming" target="_blank">HLS</a> protocol.

        user_metadata : typing.Optional[str]
            Metadata that helps you categorize your videos. You can specify a list of keys and values. Keys must be of type `string`, and values can be of the following types: `string`, `integer`, `float` or `boolean`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TasksCreateResponse]
            A video indexing task has successfully been created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tasks",
            method="POST",
            data={
                "index_id": index_id,
                "video_url": video_url,
                "enable_video_stream": enable_video_stream,
                "user_metadata": user_metadata,
            },
            files={
                **({"video_file": video_file} if video_file is not None else {}),
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TasksCreateResponse,
                    parse_obj_as(
                        type_=TasksCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def retrieve(
        self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TasksRetrieveResponse]:
        """
        This method retrieves a video indexing task.

        Parameters
        ----------
        task_id : str
            The unique identifier of the video indexing task to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TasksRetrieveResponse]
            The specified video indexing task has successfully been retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tasks/{jsonable_encoder(task_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TasksRetrieveResponse,
                    parse_obj_as(
                        type_=TasksRetrieveResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        This action cannot be undone.
        Note the following about deleting a video indexing task:
        - You can only delete video indexing tasks for which the status is `ready` or `failed`.
        - If the status of your video indexing task is `ready`, you must first delete the video vector associated with your video indexing task by calling the [`DELETE`](/v1.3/api-reference/videos/delete) method of the `/indexes/videos` endpoint.

        Parameters
        ----------
        task_id : str
            The unique identifier of the video indexing task you want to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tasks/{jsonable_encoder(task_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .base_segment import BaseSegment
from .end_offset_sec import EndOffsetSec
from .start_offset_sec import StartOffsetSec


class VideoSegment(BaseSegment):
    """
    An object that contains the video embedding and its start time. Each segment is between 2 and 10 seconds.
    """

    start_offset_sec: typing.Optional[StartOffsetSec] = None
    end_offset_sec: typing.Optional[EndOffsetSec] = None
    embedding_option: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the embedding.
    """

    embedding_scope: typing.Optional[str] = pydantic.Field(default=None)
    """
    The scope of the video embedding.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .tasks_create_request_video_embedding_scope_item import TasksCreateRequestVideoEmbeddingScopeItem
from .tasks_create_response import TasksCreateResponse
from .tasks_list_response import TasksListResponse
from .tasks_list_response_page_info import TasksListResponsePageInfo
from .tasks_retrieve_request_embedding_option_item import TasksRetrieveRequestEmbeddingOptionItem
from .tasks_retrieve_response import TasksRetrieveResponse
from .tasks_retrieve_response_video_embedding import TasksRetrieveResponseVideoEmbedding
from .tasks_status_response import TasksStatusResponse
from .tasks_status_response_video_embedding import TasksStatusResponseVideoEmbedding

__all__ = [
    "TasksCreateRequestVideoEmbeddingScopeItem",
    "TasksCreateResponse",
    "TasksListResponse",
    "TasksListResponsePageInfo",
    "TasksRetrieveRequestEmbeddingOptionItem",
    "TasksRetrieveResponse",
    "TasksRetrieveResponseVideoEmbedding",
    "TasksStatusResponse",
    "TasksStatusResponseVideoEmbedding",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.audio_input_request import AudioInputRequest
from ...types.embedding_success_response import EmbeddingSuccessResponse
from ...types.image_input_request import ImageInputRequest
from ...types.multi_input_request import MultiInputRequest
from ...types.text_image_input_request import TextImageInputRequest
from ...types.text_input_request import TextInputRequest
from ...types.video_input_request import VideoInputRequest
from .raw_client import AsyncRawV2Client, RawV2Client
from .tasks.client import AsyncTasksClient, TasksClient
from .types.create_embeddings_request_input_type import CreateEmbeddingsRequestInputType
from .types.create_embeddings_request_model_name import CreateEmbeddingsRequestModelName

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV2Client(client_wrapper=client_wrapper)
        self.tasks = TasksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV2Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV2Client
        """
        return self._raw_client

    def create(
        self,
        *,
        input_type: CreateEmbeddingsRequestInputType,
        model_name: CreateEmbeddingsRequestModelName,
        text: typing.Optional[TextInputRequest] = OMIT,
        image: typing.Optional[ImageInputRequest] = OMIT,
        text_image: typing.Optional[TextImageInputRequest] = OMIT,
        audio: typing.Optional[AudioInputRequest] = OMIT,
        video: typing.Optional[VideoInputRequest] = OMIT,
        multi_input: typing.Optional[MultiInputRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmbeddingSuccessResponse:
        """
        This endpoint synchronously creates embeddings for multimodal content and returns the results immediately in the response.

        **When to use this endpoint**:
        - Create embeddings for text, images, audio, or video content
        - Retrieve immediate results without waiting for background processing
        - Process audio or video content up to 10 minutes in duration

        **Do not use this endpoint for**:
        - Audio or video content longer than 10 minutes. Use the [`POST`](/v1.3/api-reference/create-embeddings-v2/create-async-embedding-task) method of the `/embed-v2/tasks` endpoint instead.

        <Accordion title="Input requirements">
          **Text**:
          - Maximum length: 500 tokens

          **Images**:
          - Formats: JPEG, PNG
          - Minimum size: 128x128 pixels
          - Maximum file size: 5 MB

          **Audio and video**:
          - Maximum duration: 10 minutes
          - Maximum file size for base64 encoded strings: 36 MB
          - Audio formats: WAV (uncompressed), MP3 (lossy), FLAC (lossless)
          - Video formats: [FFmpeg supported formats](https://ffmpeg.org/ffmpeg-formats.html)
          - Video resolution: 360x360 to 5184x2160 pixels
          - Aspect ratio: Between 1:1 and 1:2.4, or between 2.4:1 and 1:1
        </Accordion>

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        input_type : CreateEmbeddingsRequestInputType
            The type of content for the embeddings.


            **Values**:
            - `audio`: Creates embeddings for an audio file
            - `video`: Creates embeddings for a video file
            - `image`: Creates embeddings for an image file
            - `text`: Creates embeddings for text input
            - `text_image`: Creates embeddings for text and an image
            - `multi_input`: Creates a single embedding from up to 10 images. You can optionally include text to provide context. To reference specific images in your text, use placeholders in the following format: `<@name>`, where `name` matches the `name` field of a media source

        model_name : CreateEmbeddingsRequestModelName
            The video understanding model to use. Value: "marengo3.0".

        text : typing.Optional[TextInputRequest]

        image : typing.Optional[ImageInputRequest]

        text_image : typing.Optional[TextImageInputRequest]

        audio : typing.Optional[AudioInputRequest]

        video : typing.Optional[VideoInputRequest]

        multi_input : typing.Optional[MultiInputRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmbeddingSuccessResponse
            Successful request; normal operation

        Examples
        --------
        from twelvelabs import TextInputRequest, TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.embed.v_2.create(
            input_type="text",
            model_name="marengo3.0",
            text=TextInputRequest(
                input_text="man walking a dog",
            ),
        )
        """
        _response = self._raw_client.create(
            input_type=input_type,
            model_name=model_name,
            text=text,
            image=image,
            text_image=text_image,
            audio=audio,
            video=video,
            multi_input=multi_input,
            request_options=request_options,
        )
        return _response.data


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV2Client(client_wrapper=client_wrapper)
        self.tasks = AsyncTasksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV2Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV2Client
        """
        return self._raw_client

    async def create(
        self,
        *,
        input_type: CreateEmbeddingsRequestInputType,
        model_name: CreateEmbeddingsRequestModelName,
        text: typing.Optional[TextInputRequest] = OMIT,
        image: typing.Optional[ImageInputRequest] = OMIT,
        text_image: typing.Optional[TextImageInputRequest] = OMIT,
        audio: typing.Optional[AudioInputRequest] = OMIT,
        video: typing.Optional[VideoInputRequest] = OMIT,
        multi_input: typing.Optional[MultiInputRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmbeddingSuccessResponse:
        """
        This endpoint synchronously creates embeddings for multimodal content and returns the results immediately in the response.

        **When to use this endpoint**:
        - Create embeddings for text, images, audio, or video content
        - Retrieve immediate results without waiting for background processing
        - Process audio or video content up to 10 minutes in duration

        **Do not use this endpoint for**:
        - Audio or video content longer than 10 minutes. Use the [`POST`](/v1.3/api-reference/create-embeddings-v2/create-async-embedding-task) method of the `/embed-v2/tasks` endpoint instead.

        <Accordion title="Input requirements">
          **Text**:
          - Maximum length: 500 tokens

          **Images**:
          - Formats: JPEG, PNG
          - Minimum size: 128x128 pixels
          - Maximum file size: 5 MB

          **Audio and video**:
          - Maximum duration: 10 minutes
          - Maximum file size for base64 encoded strings: 36 MB
          - Audio formats: WAV (uncompressed), MP3 (lossy), FLAC (lossless)
          - Video formats: [FFmpeg supported formats](https://ffmpeg.org/ffmpeg-formats.html)
          - Video resolution: 360x360 to 5184x2160 pixels
          - Aspect ratio: Between 1:1 and 1:2.4, or between 2.4:1 and 1:1
        </Accordion>

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        input_type : CreateEmbeddingsRequestInputType
            The type of content for the embeddings.


            **Values**:
            - `audio`: Creates embeddings for an audio file
            - `video`: Creates embeddings for a video file
            - `image`: Creates embeddings for an image file
            - `text`: Creates embeddings for text input
            - `text_image`: Creates embeddings for text and an image
            - `multi_input`: Creates a single embedding from up to 10 images. You can optionally include text to provide context. To reference specific images in your text, use placeholders in the following format: `<@name>`, where `name` matches the `name` field of a media source

        model_name : CreateEmbeddingsRequestModelName
            The video understanding model to use. Value: "marengo3.0".

        text : typing.Optional[TextInputRequest]

        image : typing.Optional[ImageInputRequest]

        text_image : typing.Optional[TextImageInputRequest]

        audio : typing.Optional[AudioInputRequest]

        video : typing.Optional[VideoInputRequest]

        multi_input : typing.Optional[MultiInputRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmbeddingSuccessResponse
            Successful request; normal operation

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs, TextInputRequest

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.embed.v_2.create(
                input_type="text",
                model_name="marengo3.0",
                text=TextInputRequest(
                    input_text="man walking a dog",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            input_type=input_type,
            model_name=model_name,
            text=text,
            image=image,
            text_image=text_image,
            audio=audio,
            video=video,
            multi_input=multi_input,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .indexed_asset_detailed_embedding_video_embedding import IndexedAssetDetailedEmbeddingVideoEmbedding


class IndexedAssetDetailedEmbedding(UniversalBaseModel):
    """
    Contains the embedding and the associated information. The platform returns this field when the `embedding_option` parameter is specified in the request.
    """

    model_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the video understanding model used to create the embedding.
    """

    video_embedding: typing.Optional[IndexedAssetDetailedEmbeddingVideoEmbedding] = pydantic.Field(default=None)
    """
    An object that contains the embeddings.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import typing
from ..core.client_wrapper import SyncClientWrapper
from ..search.client import SearchClient
from ..types.search_item import SearchItem
from ..core.pagination import SyncPager, AsyncPager
from ..core.client_wrapper import AsyncClientWrapper
from ..core.pydantic_utilities import parse_obj_as
from ..search.client import AsyncSearchClient
from ..search.types.search_create_request_search_options_item import (
    SearchCreateRequestSearchOptionsItem,
)
from ..search.types.search_create_request_group_by import SearchCreateRequestGroupBy
from ..search.types.search_create_request_operator import SearchCreateRequestOperator
from ..types.search_results import SearchResults
from ..core.request_options import RequestOptions
from .. import core
from ..search.types.search_create_request_transcription_options_item import SearchCreateRequestTranscriptionOptionsItem

OMIT = typing.cast(typing.Any, ...)


class SearchClientWrapper(SearchClient):
    def __init__(self, client_wrapper: SyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)

    def _get_next_page(self, page_token: str) -> SyncPager[SearchItem]:
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"search/{page_token}",
            method="GET",
        )
        _parsed_response = typing.cast(
            SearchResults,
            parse_obj_as(
                type_=SearchResults,
                object_=_response.json(),
            ),
        )
        _has_next = (
            _parsed_response.page_info is not None
            and _parsed_response.page_info.next_page_token is not None
        )
        _get_next = lambda: self._get_next_page(
            _parsed_response.page_info.next_page_token  # type: ignore
        )
        _items = _parsed_response.data
        return SyncPager(
            has_next=_has_next, items=_items, get_next=_get_next, response=None
        )

    def query(
        self,
        *,
        index_id: str,
        search_options: typing.List[SearchCreateRequestSearchOptionsItem],
        query_media_type: typing.Optional[typing.Literal["image"]] = OMIT,
        query_media_url: typing.Optional[str] = OMIT,
        query_media_file: typing.Optional[core.File] = OMIT,
        query_media_urls: typing.Optional[typing.List[str]] = None,
        query_media_files: typing.Optional[typing.List[core.File]] = None,
        query_text: typing.Optional[str] = OMIT,
        adjust_confidence_level: typing.Optional[float] = OMIT,  # Deprecated: ignored by the API. Use the `rank` field in the response instead.
        group_by: typing.Optional[SearchCreateRequestGroupBy] = OMIT,
        threshold: typing.Optional[typing.Any] = OMIT,  # Deprecated: ignored by the API. Use the `rank` field in the response instead.
        sort_option: typing.Optional[str] = OMIT,  # Deprecated: ignored by the API. Use the `rank` field in the response instead.
        operator: typing.Optional[SearchCreateRequestOperator] = OMIT,
        page_limit: typing.Optional[int] = OMIT,
        filter: typing.Optional[str] = OMIT,
        include_user_metadata: typing.Optional[bool] = OMIT,
        transcription_options: typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[SearchItem]:
        """
        Use this endpoint to search for relevant matches in an index using text, media, or a combination of both as your query.

        **Text queries**:
        - Use the `query_text` parameter to specify your query.

        **Media queries**:
        - Set the `query_media_type` parameter to the corresponding media type (example: `image`).
        - Specify either one of the following parameters:
          - `query_media_url`: Publicly accessible URL of your media file.
          - `query_media_file`: Local media file.
          If both `query_media_url` and `query_media_file` are specified in the same request, `query_media_url` takes precedence.

        **Composed text and media queries** (Marengo 3.0 only):
        - Use the `query_text` parameter for your text query.
        - Set `query_media_type` to `image`.
        - Specify the image using either the `query_media_url` or the `query_media_file` parameter.

          Example: Provide an image of a car and include  "red color"  in your query to find red instances of that car model.

        <Note title="Note">
          When using images in your search queries (either as media queries or in composed searches), ensure your image files meet the [format requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements).
        </Note>

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to search.

        search_options : typing.List[SearchCreateRequestSearchOptionsItem]
            Specifies the modalities the video understanding model uses to find relevant information.

            Available options:
            - `visual`: Searches visual content.
            - `audio`: Searches non-speech audio (Marengo 3.0) or all audio (Marengo 2.7).
            - `transcription`: Spoken words (Marengo 3.0 only)

            <Note title="Note">
            - You can specify multiple search options in conjunction with the [`operator`](/v1.3/api-reference/any-to-video-search/make-search-request#request.body.operator.operator) parameter described below to broaden or narrow your search. For example, to search using both visual and non-speech audio content, include this parameter two times in the request as shown below:
              ```JSON
              --form search_options=visual \
              --form search_options=audio \
              --form search_options=transcription \
              ```
            </Note>

            For detailed guidance and version-specific behavior, see the [Search options](/v1.3/docs/concepts/modalities#search-options) section.

        query_media_type : typing.Optional[typing.Literal["image"]]
            The type of media you wish to use. This parameter is required for media queries. For example, to perform an image-based search, set this parameter to `image`. Use `query_text` together with this parameter when you want to perform a composed image+text search.

        query_media_url : typing.Optional[str]
            The publicly accessible URL of the media file you wish to use. This parameter is required for media queries if `query_media_file` is not provided.

        query_media_file : typing.Optional[core.File]
            See core.File for more documentation

        query_text : typing.Optional[str]
            The text query to search for. This parameter is required for text queries. Note that the platform supports full natural language-based search. You can use this parameter together with `query_media_type` and `query_media_url` or `query_media_file` to perform a composed image+text search.


            The maximum query length varies by model. Marengo 3.0 supports up to 500 tokens per query, while Marengo 2.7 supports up to 77 tokens per query.

        transcription_options : typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]]
            Specifies how the platform matches your text query with the words spoken in the video. This parameter applies only when using Marengo 3.0 with the `search_options` parameter containing the `transcription` value.

            Available options:
            - `lexical`: Exact word matching
            - `semantic`: Meaning-based matching

            For details on when to use each option, see the [Transcription options](/v1.3/docs/concepts/modalities#transcription-options) section.

            **Default**: `["lexical", "semantic"]`.

        adjust_confidence_level : typing.Optional[float]
            .. deprecated::
                This parameter is deprecated and ignored by the API.
                Use the `rank` field in the response instead.

        group_by : typing.Optional[SearchCreateRequestGroupBy]
            Use this parameter to group or ungroup items in a response. It can take one of the following values:
            - `video`:  The platform will group the matching video clips in the response by video.
            - `clip`: The matching video clips in the response will not be grouped.

            **Default:** `clip`

        threshold : typing.Optional[typing.Any]
            .. deprecated::
                This parameter is deprecated and ignored by the API.
                Use the `rank` field in the response instead.

        sort_option : typing.Optional[str]
            .. deprecated::
                This parameter is deprecated and ignored by the API.
                Use the `rank` field in the response instead.

        operator : typing.Optional[SearchCreateRequestOperator]
            Combines multiple search options using `or` or `and`. Use `and` to find segments matching all search options. Use `or` to find segments matching any search option. For detailed guidance on using this parameter, see the [Combine multiple modalities](/v1.3/docs/concepts/modalities#combine-multiple-modalities) section.
            
            **Default**: `or`.
        
        page_limit : typing.Optional[int]
            The number of items to return on each page. When grouping by video, this parameter represents the number of videos per page. Otherwise, it represents the maximum number of video clips per page.
            
            **Max**: `50`.
        
        filter : typing.Optional[str]
            Specifies a stringified JSON object to filter your search results. Supports both system-generated metadata (example: video ID, duration) and user-defined metadata.
            
            **Syntax for filtering**
            
            The following table describes the supported data types, operators, and filter syntax:
            
            | Data type | Operator | Description | Syntax |
            |:----------|:---------|:------------|:-------|
            | String | `=` | Matches results equal to the specified value. | `{"field": "value"}`
            | Array of strings | `=` | Matches results with any value in the specified array. Supported only for `id`. | `{"id": ["value1", "value2"]}` |
            | Numeric (integer, float) | `=`, `lte`, `gte` | Matches results equal to or within a range of the specified value. | `{"field": number}` or `{"field": { "gte": number, "lte": number }}` |
            | Boolean | `=` | Matches results equal to the specified boolean value. | `{"field": true}` or `{"field": false}`. |
            
            <br/>
            **System-generated metadata**
            
            The table below describes the system-generated metadata available for filtering your  search results:
            
            | Field name | Description | Type | Example |
            |:-----------|:------------|:-----|:--------|
            | `id` | Filters by specific video IDs. | Array of strings | `{"id": ["67cec9caf45d9b64a58340fc", "67cec9baf45d9b64a58340fa"]}`. |
            | `duration` | Filters based on the duration of the video containing the segment that matches your query. | Number or object with `gte` and `lte` | `{"duration": 600}` or `{"duration": { "gte": 600, "lte": 800 }}` |
            | `width` | Filters by video width (in pixels). | Number or object with `gte` and `lte` | `{"width": 1920}` or `{"width": { "gte": 1280, "lte": 1920}}` |
            | `height` | Filters by video height (in pixels). | Number or object with `gte` and `lte`. | `{"height": 1080}` or `{"height": { "gte": 720, "lte": 1080 }}`. |
            | `size` | Filters by video size (in bytes) | Number or object with `gte` and `lte`. | `{"size": 1048576}` or `{"size": { "gte": 1048576, "lte": 5242880}}` |
            | `filename` | Filters by the exact file name. | String | `{"filename": "Animal Encounters part 1"}` |
            
            <br/>
            **User-defined metadata**
            
            To filter by user-defined metadata:
            1. Add metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint
            2. Reference the custom field in your filter object. For example, to filter videos where a custom field named `needsReview` of type boolean is `true`, use `{"needs_review": true}`.
            
            For more details and examples, see the [Filter search results](/v1.3/docs/guides/search/filtering) page.
        
        include_user_metadata : typing.Optional[bool]
            Specifies whether to include user-defined metadata in the search results.
        
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.
        
        Returns
        -------
        SearchResults
            Successfully performed a search request.
        
        Examples
        --------
        from twelvelabs import TwelveLabs
        
        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.search.create(
            index_id="index_id",
            search_options=["visual"],
        )
        """

        _has_plural = query_media_urls is not None or query_media_files is not None
        if _has_plural:
            # Note: adjust_confidence_level, threshold, sort_option are deprecated and not sent to the API.
            _data: typing.Dict[str, typing.Any] = {
                "index_id": index_id,
                "search_options": search_options,
                "query_media_type": query_media_type,
                "query_media_url": query_media_urls if query_media_urls is not None else query_media_url,
                "query_text": query_text,
                "group_by": group_by,
                "operator": operator,
                "page_limit": page_limit,
                "filter": filter,
                "include_user_metadata": include_user_metadata,
                "transcription_options": transcription_options,
            }
            _files: typing.Dict[str, typing.Any] = {}
            if query_media_files is not None:
                _files["query_media_file"] = query_media_files
            elif query_media_file is not OMIT:
                _files["query_media_file"] = query_media_file
            _http_response = self._raw_client._client_wrapper.httpx_client.request(
                "search",
                method="POST",
                data=_data,
                files=_files,
                request_options=request_options,
                omit=OMIT,
                force_multipart=True,
            )
            _response = typing.cast(
                SearchResults,
                parse_obj_as(type_=SearchResults, object_=_http_response.json()),
            )
        else:
            # Note: adjust_confidence_level, threshold, sort_option are deprecated and not sent to the API.
            _response = self.create(
                index_id=index_id,
                search_options=search_options,
                query_media_type=query_media_type,
                query_media_url=query_media_url,
                query_media_file=query_media_file,
                query_text=query_text,
                group_by=group_by,
                operator=operator,
                page_limit=page_limit,
                filter=filter,
                include_user_metadata=include_user_metadata,
                transcription_options=transcription_options,
                request_options=request_options,
            )

        _has_next = (
            _response.page_info is not None
            and _response.page_info.next_page_token is not None
        )
        _get_next = lambda: self._get_next_page(_response.page_info.next_page_token)  # type: ignore
        _items = _response.data
        return SyncPager(
            has_next=_has_next, items=_items, get_next=_get_next, response=None
        )

    def __getattr__(self, item):
        return getattr(self.client_wrapper, item)


class AsyncSearchClientWrapper(AsyncSearchClient):
    def __init__(self, client_wrapper: AsyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)

    async def _get_next_page(self, page_token: str) -> AsyncPager[SearchItem]:
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"search/{page_token}",
            method="GET",
        )
        _parsed_response = typing.cast(
            SearchResults,
            parse_obj_as(
                type_=SearchResults,
                object_=_response.json(),
            ),
        )
        _has_next = (
            _parsed_response.page_info is not None
            and _parsed_response.page_info.next_page_token is not None
        )
        _get_next = lambda: self._get_next_page(
            _parsed_response.page_info.next_page_token  # type: ignore
        )
        _items = _parsed_response.data
        return AsyncPager(
            has_next=_has_next, items=_items, get_next=_get_next, response=None
        )

    async def query(
        self,
        *,
        index_id: str,
        search_options: typing.List[SearchCreateRequestSearchOptionsItem],
        query_media_type: typing.Optional[typing.Literal["image"]] = OMIT,
        query_media_url: typing.Optional[str] = OMIT,
        query_media_file: typing.Optional[core.File] = OMIT,
        query_media_urls: typing.Optional[typing.List[str]] = None,
        query_media_files: typing.Optional[typing.List[core.File]] = None,
        query_text: typing.Optional[str] = OMIT,
        adjust_confidence_level: typing.Optional[float] = OMIT,  # Deprecated: ignored by the API. Use the `rank` field in the response instead.
        group_by: typing.Optional[SearchCreateRequestGroupBy] = OMIT,
        threshold: typing.Optional[typing.Any] = OMIT,  # Deprecated: ignored by the API. Use the `rank` field in the response instead.
        sort_option: typing.Optional[str] = OMIT,  # Deprecated: ignored by the API. Use the `rank` field in the response instead.
        operator: typing.Optional[SearchCreateRequestOperator] = OMIT,
        page_limit: typing.Optional[int] = OMIT,
        filter: typing.Optional[str] = OMIT,
        include_user_metadata: typing.Optional[bool] = OMIT,
        transcription_options: typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[SearchItem]:
        """
        Use this endpoint to search for relevant matches in an index using text, media, or a combination of both as your query.

        **Text queries**:
        - Use the `query_text` parameter to specify your query.

        **Media queries**:
        - Set the `query_media_type` parameter to the corresponding media type (example: `image`).
        - Specify either one of the following parameters:
          - `query_media_url`: Publicly accessible URL of your media file.
          - `query_media_file`: Local media file.
          If both `query_media_url` and `query_media_file` are specified in the same request, `query_media_url` takes precedence.

        **Composed text and media queries** (Marengo 3.0 only):
        - Use the `query_text` parameter for your text query.
        - Set `query_media_type` to `image`.
        - Specify the image using either the `query_media_url` or the `query_media_file` parameter.

          Example: Provide an image of a car and include  "red color"  in your query to find red instances of that car model.

        <Note title="Note">
          When using images in your search queries (either as media queries or in composed searches), ensure your image files meet the [format requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements).
        </Note>

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to search.

        search_options : typing.List[SearchCreateRequestSearchOptionsItem]
            Specifies the modalities the video understanding model uses to find relevant information.

            Available options:
            - `visual`: Searches visual content.
            - `audio`: Searches non-speech audio (Marengo 3.0) or all audio (Marengo 2.7).
            - `transcription`: Spoken words (Marengo 3.0 only)

            <Note title="Note">
            - You can specify multiple search options in conjunction with the [`operator`](/v1.3/api-reference/any-to-video-search/make-search-request#request.body.operator.operator) parameter described below to broaden or narrow your search. For example, to search using both visual and non-speech audio content, include this parameter two times in the request as shown below:
              ```JSON
              --form search_options=visual \
              --form search_options=audio \
              --form search_options=transcription \
              ```
            </Note>

            For detailed guidance and version-specific behavior, see the [Search options](/v1.3/docs/concepts/modalities#search-options) section.

        query_media_type : typing.Optional[typing.Literal["image"]]
            The type of media you wish to use. This parameter is required for media queries. For example, to perform an image-based search, set this parameter to `image`. Use `query_text` together with this parameter when you want to perform a composed image+text search.

        query_media_url : typing.Optional[str]
            The publicly accessible URL of the media file you wish to use. This parameter is required for media queries if `query_media_file` is not provided.

        query_media_file : typing.Optional[core.File]
            See core.File for more documentation

        query_text : typing.Optional[str]
            The text query to search for. This parameter is required for text queries. Note that the platform supports full natural language-based search. You can use this parameter together with `query_media_type` and `query_media_url` or `query_media_file` to perform a composed image+text search.


            The maximum query length varies by model. Marengo 3.0 supports up to 500 tokens per query, while Marengo 2.7 supports up to 77 tokens per query.

        transcription_options : typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]]
            Specifies how the platform matches your text query with the words spoken in the video. This parameter applies only when using Marengo 3.0 with the `search_options` parameter containing the `transcription` value.

            Available options:
            - `lexical`: Exact word matching
            - `semantic`: Meaning-based matching

            For details on when to use each option, see the [Transcription options](/v1.3/docs/concepts/modalities#transcription-options) section.

            **Default**: `["lexical", "semantic"]`.

        adjust_confidence_level : typing.Optional[float]
            .. deprecated::
                This parameter is deprecated and ignored by the API.
                Use the `rank` field in the response instead.

        group_by : typing.Optional[SearchCreateRequestGroupBy]
            Use this parameter to group or ungroup items in a response. It can take one of the following values:
            - `video`:  The platform will group the matching video clips in the response by video.
            - `clip`: The matching video clips in the response will not be grouped.

            **Default:** `clip`

        threshold : typing.Optional[typing.Any]
            .. deprecated::
                This parameter is deprecated and ignored by the API.
                Use the `rank` field in the response instead.

        sort_option : typing.Optional[str]
            .. deprecated::
                This parameter is deprecated and ignored by the API.
                Use the `rank` field in the response instead.

        operator : typing.Optional[SearchCreateRequestOperator]
            Combines multiple search options using `or` or `and`. Use `and` to find segments matching all search options. Use `or` to find segments matching any search option. For detailed guidance on using this parameter, see the [Combine multiple modalities](/v1.3/docs/concepts/modalities#combine-multiple-modalities) section.

            **Default**: `or`.

        page_limit : typing.Optional[int]
            The number of items to return on each page. When grouping by video, this parameter represents the number of videos per page. Otherwise, it represents the maximum number of video clips per page.

            **Max**: `50`.

        filter : typing.Optional[str]
            Specifies a stringified JSON object to filter your search results. Supports both system-generated metadata (example: video ID, duration) and user-defined metadata.

            **Syntax for filtering**

            The following table describes the supported data types, operators, and filter syntax:

            | Data type | Operator | Description | Syntax |
            |:----------|:---------|:------------|:-------|
            | String | `=` | Matches results equal to the specified value. | `{"field": "value"}`
            | Array of strings | `=` | Matches results with any value in the specified array. Supported only for `id`. | `{"id": ["value1", "value2"]}` |
            | Numeric (integer, float) | `=`, `lte`, `gte` | Matches results equal to or within a range of the specified value. | `{"field": number}` or `{"field": { "gte": number, "lte": number }}` |
            | Boolean | `=` | Matches results equal to the specified boolean value. | `{"field": true}` or `{"field": false}`. |

            <br/>
            **System-generated metadata**

            The table below describes the system-generated metadata available for filtering your  search results:

            | Field name | Description | Type | Example |
            |:-----------|:------------|:-----|:--------|
            | `id` | Filters by specific video IDs. | Array of strings | `{"id": ["67cec9caf45d9b64a58340fc", "67cec9baf45d9b64a58340fa"]}`. |
            | `duration` | Filters based on the duration of the video containing the segment that matches your query. | Number or object with `gte` and `lte` | `{"duration": 600}` or `{"duration": { "gte": 600, "lte": 800 }}` |
            | `width` | Filters by video width (in pixels). | Number or object with `gte` and `lte` | `{"width": 1920}` or `{"width": { "gte": 1280, "lte": 1920}}` |
            | `height` | Filters by video height (in pixels). | Number or object with `gte` and `lte`. | `{"height": 1080}` or `{"height": { "gte": 720, "lte": 1080 }}`. |
            | `size` | Filters by video size (in bytes) | Number or object with `gte` and `lte`. | `{"size": 1048576}` or `{"size": { "gte": 1048576, "lte": 5242880}}` |
            | `filename` | Filters by the exact file name. | String | `{"filename": "Animal Encounters part 1"}` |

            <br/>
            **User-defined metadata**

            To filter by user-defined metadata:
            1. Add metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint
            2. Reference the custom field in your filter object. For example, to filter videos where a custom field named `needsReview` of type boolean is `true`, use `{"needs_review": true}`.

            For more details and examples, see the [Filter search results](/v1.3/docs/guides/search/filtering) page.

        include_user_metadata : typing.Optional[bool]
            Specifies whether to include user-defined metadata in the search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchResults
            Successfully performed a search request.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.search.create(
                index_id="index_id",
                search_options=["visual"],
            )


        asyncio.run(main())
        """

        _has_plural = query_media_urls is not None or query_media_files is not None
        if _has_plural:
            # Note: adjust_confidence_level, threshold, sort_option are deprecated and not sent to the API.
            _data: typing.Dict[str, typing.Any] = {
                "index_id": index_id,
                "search_options": search_options,
                "query_media_type": query_media_type,
                "query_media_url": query_media_urls if query_media_urls is not None else query_media_url,
                "query_text": query_text,
                "group_by": group_by,
                "operator": operator,
                "page_limit": page_limit,
                "filter": filter,
                "include_user_metadata": include_user_metadata,
                "transcription_options": transcription_options,
            }
            _files: typing.Dict[str, typing.Any] = {}
            if query_media_files is not None:
                _files["query_media_file"] = query_media_files
            elif query_media_file is not OMIT:
                _files["query_media_file"] = query_media_file
            _http_response = await self._raw_client._client_wrapper.httpx_client.request(
                "search",
                method="POST",
                data=_data,
                files=_files,
                request_options=request_options,
                omit=OMIT,
                force_multipart=True,
            )
            _response = typing.cast(
                SearchResults,
                parse_obj_as(type_=SearchResults, object_=_http_response.json()),
            )
        else:
            # Note: adjust_confidence_level, threshold, sort_option are deprecated and not sent to the API.
            _response = await self.create(
                index_id=index_id,
                search_options=search_options,
                query_media_type=query_media_type,
                query_media_url=query_media_url,
                query_media_file=query_media_file,
                query_text=query_text,
                group_by=group_by,
                operator=operator,
                page_limit=page_limit,
                filter=filter,
                include_user_metadata=include_user_metadata,
                transcription_options=transcription_options,
                request_options=request_options,
            )

        _has_next = (
            _response.page_info is not None
            and _response.page_info.next_page_token is not None
        )
        _get_next = lambda: self._get_next_page(_response.page_info.next_page_token)  # type: ignore
        _items = _response.data
        return AsyncPager(
            has_next=_has_next, items=_items, get_next=_get_next, response=None
        )

    def __getattr__(self, item):
        return getattr(self.client_wrapper, item)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .index_models_item import IndexModelsItem


class IndexSchema(UniversalBaseModel):
    """
    An index groups one or more videos stored as vectors and is the most granular level at which you can perform a search.
    """

    id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="_id")] = pydantic.Field(default=None)
    """
    A string representing the unique identifier of the index. It is assigned by the platform when an index is created.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string representing the date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), that the index was created.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string representing the date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), that the index has been updated.
    """

    expires_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string representing the date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when your index will expire.
    
    If you're on the Free plan, the platform retains your index data for 90 days from creation. After this period, the platform deletes your index data, and this action cannot be undone. To continue using your index beyond this period, consider upgrading to the Developer plan, which offers unlimited index retention. For details, see the [Upgrade your plan](/v1.3/docs/get-started/manage-your-plan#upgrade-your-plan) section.
    
    If you're on the Developer plan, this field is set to `null`, indicating no expiration.
    """

    index_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string representing the name of the index.
    """

    total_duration: typing.Optional[float] = pydantic.Field(default=None)
    """
    A number representing the total duration, in seconds, of the videos in the index.
    """

    video_count: typing.Optional[float] = pydantic.Field(default=None)
    """
    The number of videos uploaded to this index.
    """

    models: typing.Optional[typing.List[IndexModelsItem]] = pydantic.Field(default=None)
    """
    An array containing the list of the [video understanding models](/v1.3/docs/concepts/models) enabled for this index.
    """

    addons: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of the add-ons that are enabled for this index.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .create_embeddings_request_input_type import CreateEmbeddingsRequestInputType
from .create_embeddings_request_model_name import CreateEmbeddingsRequestModelName

__all__ = ["CreateEmbeddingsRequestInputType", "CreateEmbeddingsRequestModelName"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .end_time import EndTime
from .rank import Rank
from .start_time import StartTime
from .thumbnail_url import ThumbnailUrl
from .user_metadata import UserMetadata


class SearchItemClipsItem(UniversalBaseModel):
    start: typing.Optional[StartTime] = None
    end: typing.Optional[EndTime] = None
    rank: typing.Optional[Rank] = None
    thumbnail_url: typing.Optional[ThumbnailUrl] = None
    transcription: typing.Optional[str] = pydantic.Field(default=None)
    """
    A transcription of the spoken words that are captured in the clip.
    """

    video_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string representing the unique identifier of the video for the corresponding clip.
    """

    user_metadata: typing.Optional[UserMetadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .base_segment import BaseSegment


class TextEmbeddingResult(UniversalBaseModel):
    """
    An object containing the generated embedding vector.
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if the embedding generation failed.
    """

    segments: typing.Optional[typing.List[BaseSegment]] = pydantic.Field(default=None)
    """
    An object that contains the embedding.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.index_schema import IndexSchema
from .indexed_assets.client import AsyncIndexedAssetsClient, IndexedAssetsClient
from .raw_client import AsyncRawIndexesClient, RawIndexesClient
from .types.indexes_create_request_models_item import IndexesCreateRequestModelsItem
from .types.indexes_create_response import IndexesCreateResponse
from .videos.client import AsyncVideosClient, VideosClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class IndexesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawIndexesClient(client_wrapper=client_wrapper)
        self.indexed_assets = IndexedAssetsClient(client_wrapper=client_wrapper)

        self.videos = VideosClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawIndexesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawIndexesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        index_name: typing.Optional[str] = None,
        model_options: typing.Optional[str] = None,
        model_family: typing.Optional[str] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[IndexSchema]:
        """
        This method returns a list of the indexes in your account. The platform returns indexes sorted by creation date, with the oldest indexes at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        index_name : typing.Optional[str]
            Filter by the name of an index.

        model_options : typing.Optional[str]
            Filter by the model options. When filtering by multiple model options, the values must be comma-separated.

        model_family : typing.Optional[str]
            Filter by the model family. This parameter can take one of the following values: `marengo` or `pegasus`. You can specify a single value.

        created_at : typing.Optional[str]
            Filter indexes by the creation date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the indexes that were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            Filter indexes by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the indexes that were last updated on the specified date at or after the given time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[IndexSchema]
            The indexes have successfully been retrieved.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        response = client.indexes.list(
            page=1,
            page_limit=10,
            sort_by="created_at",
            sort_option="desc",
            index_name="myIndex",
            model_options="visual,audio",
            model_family="marengo",
            created_at="2024-08-16T16:53:59Z",
            updated_at="2024-08-16T16:55:59Z",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page,
            page_limit=page_limit,
            sort_by=sort_by,
            sort_option=sort_option,
            index_name=index_name,
            model_options=model_options,
            model_family=model_family,
            created_at=created_at,
            updated_at=updated_at,
            request_options=request_options,
        )

    def create(
        self,
        *,
        index_name: str,
        models: typing.Sequence[IndexesCreateRequestModelsItem],
        addons: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IndexesCreateResponse:
        """
        This method creates an index.

        Parameters
        ----------
        index_name : str
            The name of the index. Make sure you use a succinct and descriptive name.

        models : typing.Sequence[IndexesCreateRequestModelsItem]
            An array that specifies the [video understanding models](/v1.3/docs/concepts/models) and the [model options](/v1.3/docs/concepts/modalities#model-options) to be enabled for this index. Models determine what tasks you can perform with your videos. Model options determine which modalities the platform analyzes.

        addons : typing.Optional[typing.Sequence[str]]
            An array specifying which add-ons should be enabled. Each entry in the array is an addon, and the following values are supported:
            - `thumbnail`: Enables thumbnail generation.

            If you don't provide this parameter, no add-ons will be enabled.

            <Note title="Notes">
            - You can only enable addons when using the Marengo video understanding model.
            - You cannot disable an add-on once the index has been created.
            </Note>

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IndexesCreateResponse
            An index has successfully been created

        Examples
        --------
        from twelvelabs import TwelveLabs
        from twelvelabs.indexes import IndexesCreateRequestModelsItem

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.indexes.create(
            index_name="myIndex",
            models=[
                IndexesCreateRequestModelsItem(
                    model_name="marengo3.0",
                    model_options=["visual", "audio"],
                ),
                IndexesCreateRequestModelsItem(
                    model_name="pegasus1.2",
                    model_options=["visual", "audio"],
                ),
            ],
            addons=["thumbnail"],
        )
        """
        _response = self._raw_client.create(
            index_name=index_name, models=models, addons=addons, request_options=request_options
        )
        return _response.data

    def retrieve(self, index_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> IndexSchema:
        """
        This method retrieves details about the specified index.

        Parameters
        ----------
        index_id : str
            Unique identifier of the index to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IndexSchema
            The specified index has successfully been retrieved.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.indexes.retrieve(
            index_id="6298d673f1090f1100476d4c",
        )
        """
        _response = self._raw_client.retrieve(index_id, request_options=request_options)
        return _response.data

    def update(
        self, index_id: str, *, index_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        This method updates the name of the specified index.

        Parameters
        ----------
        index_id : str
            Unique identifier of the index to update.

        index_name : str
            The name of the index.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.indexes.update(
            index_id="6298d673f1090f1100476d4c",
            index_name="myIndex",
        )
        """
        _response = self._raw_client.update(index_id, index_name=index_name, request_options=request_options)
        return _response.data

    def delete(self, index_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        This method deletes the specified index and all the videos within it. This action cannot be undone.

        Parameters
        ----------
        index_id : str
            Unique identifier of the index to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.indexes.delete(
            index_id="6298d673f1090f1100476d4c",
        )
        """
        _response = self._raw_client.delete(index_id, request_options=request_options)
        return _response.data


class AsyncIndexesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawIndexesClient(client_wrapper=client_wrapper)
        self.indexed_assets = AsyncIndexedAssetsClient(client_wrapper=client_wrapper)

        self.videos = AsyncVideosClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawIndexesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawIndexesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        index_name: typing.Optional[str] = None,
        model_options: typing.Optional[str] = None,
        model_family: typing.Optional[str] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[IndexSchema]:
        """
        This method returns a list of the indexes in your account. The platform returns indexes sorted by creation date, with the oldest indexes at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        index_name : typing.Optional[str]
            Filter by the name of an index.

        model_options : typing.Optional[str]
            Filter by the model options. When filtering by multiple model options, the values must be comma-separated.

        model_family : typing.Optional[str]
            Filter by the model family. This parameter can take one of the following values: `marengo` or `pegasus`. You can specify a single value.

        created_at : typing.Optional[str]
            Filter indexes by the creation date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the indexes that were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            Filter indexes by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the indexes that were last updated on the specified date at or after the given time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[IndexSchema]
            The indexes have successfully been retrieved.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.indexes.list(
                page=1,
                page_limit=10,
                sort_by="created_at",
                sort_option="desc",
                index_name="myIndex",
                model_options="visual,audio",
                model_family="marengo",
                created_at="2024-08-16T16:53:59Z",
                updated_at="2024-08-16T16:55:59Z",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page,
            page_limit=page_limit,
            sort_by=sort_by,
            sort_option=sort_option,
            index_name=index_name,
            model_options=model_options,
            model_family=model_family,
            created_at=created_at,
            updated_at=updated_at,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        index_name: str,
        models: typing.Sequence[IndexesCreateRequestModelsItem],
        addons: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IndexesCreateResponse:
        """
        This method creates an index.

        Parameters
        ----------
        index_name : str
            The name of the index. Make sure you use a succinct and descriptive name.

        models : typing.Sequence[IndexesCreateRequestModelsItem]
            An array that specifies the [video understanding models](/v1.3/docs/concepts/models) and the [model options](/v1.3/docs/concepts/modalities#model-options) to be enabled for this index. Models determine what tasks you can perform with your videos. Model options determine which modalities the platform analyzes.

        addons : typing.Optional[typing.Sequence[str]]
            An array specifying which add-ons should be enabled. Each entry in the array is an addon, and the following values are supported:
            - `thumbnail`: Enables thumbnail generation.

            If you don't provide this parameter, no add-ons will be enabled.

            <Note title="Notes">
            - You can only enable addons when using the Marengo video understanding model.
            - You cannot disable an add-on once the index has been created.
            </Note>

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IndexesCreateResponse
            An index has successfully been created

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs
        from twelvelabs.indexes import IndexesCreateRequestModelsItem

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.indexes.create(
                index_name="myIndex",
                models=[
                    IndexesCreateRequestModelsItem(
                        model_name="marengo3.0",
                        model_options=["visual", "audio"],
                    ),
                    IndexesCreateRequestModelsItem(
                        model_name="pegasus1.2",
                        model_options=["visual", "audio"],
                    ),
                ],
                addons=["thumbnail"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            index_name=index_name, models=models, addons=addons, request_options=request_options
        )
        return _response.data

    async def retrieve(self, index_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> IndexSchema:
        """
        This method retrieves details about the specified index.

        Parameters
        ----------
        index_id : str
            Unique identifier of the index to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IndexSchema
            The specified index has successfully been retrieved.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.indexes.retrieve(
                index_id="6298d673f1090f1100476d4c",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(index_id, request_options=request_options)
        return _response.data

    async def update(
        self, index_id: str, *, index_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        This method updates the name of the specified index.

        Parameters
        ----------
        index_id : str
            Unique identifier of the index to update.

        index_name : str
            The name of the index.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.indexes.update(
                index_id="6298d673f1090f1100476d4c",
                index_name="myIndex",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(index_id, index_name=index_name, request_options=request_options)
        return _response.data

    async def delete(self, index_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        This method deletes the specified index and all the videos within it. This action cannot be undone.

        Parameters
        ----------
        index_id : str
            Unique identifier of the index to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.indexes.delete(
                index_id="6298d673f1090f1100476d4c",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(index_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .embedding_audio_metadata_embedding_scopes_item import EmbeddingAudioMetadataEmbeddingScopesItem


class EmbeddingAudioMetadata(UniversalBaseModel):
    """
    Metadata for audio embeddings
    """

    input_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The publicly accessible URL for the audio file
    """

    input_filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the audio file
    """

    embedding_options: typing.List[str] = pydantic.Field()
    """
    Audio embedding options used for generating the embedding
    """

    embedding_scopes: typing.List[EmbeddingAudioMetadataEmbeddingScopesItem] = pydantic.Field()
    """
    Audio embedding scopes
    """

    duration: float = pydantic.Field()
    """
    Duration of the audio in seconds
    """

    start_offset_sec: typing.Optional[float] = pydantic.Field(default=None)
    """
    Start offset in seconds
    """

    end_offset_sec: typing.Optional[float] = pydantic.Field(default=None)
    """
    End offset in seconds
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import typing
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import SyncClientWrapper, AsyncClientWrapper
from ..core.http_response import BaseHttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import SyncPager, AsyncPager
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..types.video_vector import VideoVector
from ..indexes.client import IndexesClient, AsyncIndexesClient
from ..indexes.videos.client import VideosClient, AsyncVideosClient
from ..indexes.videos.types.videos_list_request_user_metadata_value import (
    VideosListRequestUserMetadataValue,
)
from ..indexes.videos.types.videos_list_response import VideosListResponse
from ..errors.bad_request_error import BadRequestError

OMIT = typing.cast(typing.Any, ...)


class VideosClientWrapper(VideosClient):
    """Wrapper for the VideosClient that adds additional functionality."""

    def __init__(self, client_wrapper: SyncClientWrapper):
        """Initialize the VideosClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)

    def list(
        self,
        index_id: str,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        filename: typing.Optional[str] = None,
        duration: typing.Optional[float] = None,
        fps: typing.Optional[float] = None,
        width: typing.Optional[float] = None,
        height: typing.Optional[int] = None,
        size: typing.Optional[float] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        user_metadata: typing.Optional[
            typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[VideoVector]:
        """
        This method returns a list of the videos in the specified index. By default, the API returns your videos sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index for which the API will retrieve the videos.

        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        filename : typing.Optional[str]
            Filter by filename.

        duration : typing.Optional[float]
            Filter by duration. Expressed in seconds.

        fps : typing.Optional[float]
            Filter by frames per second.

        width : typing.Optional[float]
            Filter by width.

        height : typing.Optional[int]
            Filter by height.

        size : typing.Optional[float]
            Filter by size. Expressed in bytes.

        created_at : typing.Optional[str]
            Filter videos by the creation date and time of their associated indexing tasks, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the videos whose indexing tasks were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            This filter applies only to videos updated using the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/{index-id}/videos/{video-id}` endpoint. It filters videos by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were last updated on the specified date at or after the given time.

        user_metadata : typing.Optional[typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]]
            To enable filtering by custom fields, you must first add user-defined metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint.

            Examples:
            - To filter on a string: `?category=recentlyAdded`
            - To filter on an integer: `?batchNumber=5`
            - To filter on a float: `?rating=9.3`
            - To filter on a boolean: `?needsReview=true`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[VideoVector]
            The video vectors in the specified index have successfully been retrieved.

        Examples
        --------
        from twelvelabs import TwelveLabs
        client = TwelveLabs(api_key="YOUR_API_KEY", )
        response = client.indexes.videos.list(index_id='6298d673f1090f1100476d4c', page=1, page_limit=10, sort_by='created_at', sort_option='desc', filename='01.mp4', created_at='2024-08-16T16:53:59Z', updated_at='2024-08-16T16:53:59Z', )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        page = page if page is not None else 1

        # Build params dict
        params = {
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
            "filename": filename,
            "duration": duration,
            "fps": fps,
            "width": width,
            "height": height,
            "size": size,
            "created_at": created_at,
            "updated_at": updated_at,
        }

        # Flatten user_metadata into root level parameters
        if user_metadata:
            for key, value in user_metadata.items():
                params[key] = value

        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"indexes/{jsonable_encoder(index_id)}/videos",
            method="GET",
            params=params,
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    VideosListResponse,
                    parse_obj_as(
                        type_=VideosListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True
                _get_next = lambda: self.list(
                    index_id,
                    page=page + 1,
                    page_limit=page_limit,
                    sort_by=sort_by,
                    sort_option=sort_option,
                    filename=filename,
                    duration=duration,
                    fps=fps,
                    width=width,
                    height=height,
                    size=size,
                    created_at=created_at,
                    updated_at=updated_at,
                    user_metadata=user_metadata,
                    request_options=request_options,
                )
                return SyncPager(
                    has_next=_has_next,
                    items=_items,
                    get_next=_get_next,
                    response=BaseHttpResponse(response=_response),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(
                status_code=_response.status_code,
                headers=dict(_response.headers),
                body=_response.text,
            )
        raise ApiError(
            status_code=_response.status_code,
            headers=dict(_response.headers),
            body=_response_json,
        )


class AsyncVideosClientWrapper(AsyncVideosClient):
    """Async wrapper for the VideosClient that adds additional functionality."""

    def __init__(self, client_wrapper: AsyncClientWrapper):
        """Initialize the AsyncVideosClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)

    async def list(
        self,
        index_id: str,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        filename: typing.Optional[str] = None,
        duration: typing.Optional[float] = None,
        fps: typing.Optional[float] = None,
        width: typing.Optional[float] = None,
        height: typing.Optional[int] = None,
        size: typing.Optional[float] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        user_metadata: typing.Optional[
            typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[VideoVector]:
        """
        This method returns a list of the videos in the specified index. By default, the API returns your videos sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index for which the API will retrieve the videos.

        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        filename : typing.Optional[str]
            Filter by filename.

        duration : typing.Optional[float]
            Filter by duration. Expressed in seconds.

        fps : typing.Optional[float]
            Filter by frames per second.

        width : typing.Optional[float]
            Filter by width.

        height : typing.Optional[int]
            Filter by height.

        size : typing.Optional[float]
            Filter by size. Expressed in bytes.

        created_at : typing.Optional[str]
            Filter videos by the creation date and time of their associated indexing tasks, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the videos whose indexing tasks were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            This filter applies only to videos updated using the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/{index-id}/videos/{video-id}` endpoint. It filters videos by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were last updated on the specified date at or after the given time.

        user_metadata : typing.Optional[typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]]
            To enable filtering by custom fields, you must first add user-defined metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint.

            Examples:
            - To filter on a string: `?category=recentlyAdded`
            - To filter on an integer: `?batchNumber=5`
            - To filter on a float: `?rating=9.3`
            - To filter on a boolean: `?needsReview=true`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[VideoVector]
            The video vectors in the specified index have successfully been retrieved.

        Examples
        --------
        from twelvelabs import AsyncTwelveLabs
        import asyncio
        client = AsyncTwelveLabs(api_key="YOUR_API_KEY", )
        async def main() -> None:
            response = await client.indexes.videos.list(index_id='6298d673f1090f1100476d4c', page=1, page_limit=10, sort_by='created_at', sort_option='desc', filename='01.mp4', created_at='2024-08-16T16:53:59Z', updated_at='2024-08-16T16:53:59Z', )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page
        asyncio.run(main())
        """
        page = page if page is not None else 1

        # Build params dict
        params = {
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
            "filename": filename,
            "duration": duration,
            "fps": fps,
            "width": width,
            "height": height,
            "size": size,
            "created_at": created_at,
            "updated_at": updated_at,
        }

        # Flatten user_metadata into root level parameters
        if user_metadata:
            for key, value in user_metadata.items():
                params[key] = value

        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"indexes/{jsonable_encoder(index_id)}/videos",
            method="GET",
            params=params,
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    VideosListResponse,
                    parse_obj_as(
                        type_=VideosListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True

                async def _get_next():
                    return await self.list(
                        index_id,
                        page=page + 1,
                        page_limit=page_limit,
                        sort_by=sort_by,
                        sort_option=sort_option,
                        filename=filename,
                        duration=duration,
                        fps=fps,
                        width=width,
                        height=height,
                        size=size,
                        created_at=created_at,
                        updated_at=updated_at,
                        user_metadata=user_metadata,
                        request_options=request_options,
                    )

                return AsyncPager(
                    has_next=_has_next,
                    items=_items,
                    get_next=_get_next,
                    response=BaseHttpResponse(response=_response),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(
                status_code=_response.status_code,
                headers=dict(_response.headers),
                body=_response.text,
            )
        raise ApiError(
            status_code=_response.status_code,
            headers=dict(_response.headers),
            body=_response_json,
        )


class IndexesClientWrapper(IndexesClient):
    """Wrapper for the IndexesClient that adds custom functionality."""

    def __init__(self, client_wrapper: SyncClientWrapper):
        """Initialize the IndexesClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)
        # Replace the videos property with our custom implementation
        self.videos = VideosClientWrapper(client_wrapper=client_wrapper)


class AsyncIndexesClientWrapper(AsyncIndexesClient):
    """Async wrapper for the IndexesClient that adds custom functionality."""

    def __init__(self, client_wrapper: AsyncClientWrapper):
        """Initialize the AsyncIndexesClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)
        # Replace the videos property with our custom implementation
        self.videos = AsyncVideosClientWrapper(client_wrapper=client_wrapper)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....types.video_segment import VideoSegment


class VideosRetrieveResponseEmbeddingVideoEmbedding(UniversalBaseModel):
    """
    An object that contains the embeddings.
    """

    segments: typing.Optional[typing.List[VideoSegment]] = pydantic.Field(default=None)
    """
    An array of objects that contains the embeddings for each individual segment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.conflict_error import ConflictError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_found_error import NotFoundError
from ...types.analyze_max_tokens import AnalyzeMaxTokens
from ...types.analyze_task_response import AnalyzeTaskResponse
from ...types.analyze_task_status import AnalyzeTaskStatus
from ...types.analyze_temperature import AnalyzeTemperature
from ...types.analyze_text_prompt import AnalyzeTextPrompt
from ...types.create_analyze_task_response import CreateAnalyzeTaskResponse
from ...types.error_response import ErrorResponse
from ...types.response_format import ResponseFormat
from ...types.video_context import VideoContext
from .types.tasks_list_response import TasksListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTasksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        status: typing.Optional[AnalyzeTaskStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TasksListResponse]:
        """
        This method returns a list of the analysis tasks in your account. The platform returns your analysis tasks sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        status : typing.Optional[AnalyzeTaskStatus]
            Filter analysis tasks by status.
            Possible values: `queued`, `pending`, `processing`, `ready`, `failed`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TasksListResponse]
            A list of analysis tasks has successfully been retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "analyze/tasks",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
                "status": status,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TasksListResponse,
                    parse_obj_as(
                        type_=TasksListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        video: VideoContext,
        prompt: AnalyzeTextPrompt,
        temperature: typing.Optional[AnalyzeTemperature] = OMIT,
        max_tokens: typing.Optional[AnalyzeMaxTokens] = OMIT,
        response_format: typing.Optional[ResponseFormat] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateAnalyzeTaskResponse]:
        """
        This method asynchronously analyzes your videos and generates fully customizable text based on your prompts.

        <Accordion title="Input requirements">
        - Minimum duration: 4 seconds
        - Maximum duration: 2 hours
        - Formats: [FFmpeg supported formats](https://ffmpeg.org/ffmpeg-formats.html)
        - Resolution: 360x360 to 5184x2160 pixels
        - Aspect ratio: Between 1:1 and 1:2.4, or between 2.4:1 and 1:1.
        </Accordion>

        **When to use this method**:
        - Analyze videos longer than 1 hour
        - Process videos asynchronously without blocking your application

        **Do not use this method for**:
        - Videos for which you need immediate results or real-time streaming. Use the [`POST`](/v1.3/api-reference/analyze-videos/sync-analysis) method of the `/analyze` endpoint instead.

        Analyzing videos asynchronously requires three steps:

        1. Create an analysis task using this method. The platform returns a task ID.
        2. Poll the status of the task using the [`GET`](/v1.3/api-reference/analyze-videos/retrieve-analysis-task-status-results) method of the `/analyze/tasks/{task_id}` endpoint. Wait until the status is `ready`.
        3. Retrieve the results from the response when the status is `ready` using the [`GET`](/v1.3/api-reference/analyze-videos/retrieve-analysis-task-status-results) method of the `/analyze/tasks/{task_id}` endpoint.

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        video : VideoContext

        prompt : AnalyzeTextPrompt

        temperature : typing.Optional[AnalyzeTemperature]

        max_tokens : typing.Optional[AnalyzeMaxTokens]

        response_format : typing.Optional[ResponseFormat]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateAnalyzeTaskResponse]
            An analysis task has successfully been created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "analyze/tasks",
            method="POST",
            json={
                "video": convert_and_respect_annotation_metadata(
                    object_=video, annotation=VideoContext, direction="write"
                ),
                "prompt": prompt,
                "temperature": temperature,
                "max_tokens": max_tokens,
                "response_format": convert_and_respect_annotation_metadata(
                    object_=response_format, annotation=ResponseFormat, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAnalyzeTaskResponse,
                    parse_obj_as(
                        type_=CreateAnalyzeTaskResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def retrieve(
        self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AnalyzeTaskResponse]:
        """
        This method retrieves the status and results of an analysis task.

        **Task statuses**:
        - `queued`: The task is waiting to be processed.
        - `pending`: The task is queued and waiting to start.
        - `processing`: The platform is analyzing the video.
        - `ready`: Processing is complete. Results are available in the response.
        - `failed`: The task failed. No results were generated.

        Poll this method until `status` is `ready` or `failed`. When `status` is `ready`, use the results from the response.

        Parameters
        ----------
        task_id : str
            The unique identifier of the analysis task.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AnalyzeTaskResponse]
            Task status and results retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"analyze/tasks/{jsonable_encoder(task_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AnalyzeTaskResponse,
                    parse_obj_as(
                        type_=AnalyzeTaskResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        This method deletes an analysis task. You can only delete tasks that are not currently being processed.

        Parameters
        ----------
        task_id : str
            The unique identifier of the analyze task.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"analyze/tasks/{jsonable_encoder(task_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTasksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        status: typing.Optional[AnalyzeTaskStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TasksListResponse]:
        """
        This method returns a list of the analysis tasks in your account. The platform returns your analysis tasks sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        status : typing.Optional[AnalyzeTaskStatus]
            Filter analysis tasks by status.
            Possible values: `queued`, `pending`, `processing`, `ready`, `failed`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TasksListResponse]
            A list of analysis tasks has successfully been retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "analyze/tasks",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
                "status": status,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TasksListResponse,
                    parse_obj_as(
                        type_=TasksListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        video: VideoContext,
        prompt: AnalyzeTextPrompt,
        temperature: typing.Optional[AnalyzeTemperature] = OMIT,
        max_tokens: typing.Optional[AnalyzeMaxTokens] = OMIT,
        response_format: typing.Optional[ResponseFormat] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateAnalyzeTaskResponse]:
        """
        This method asynchronously analyzes your videos and generates fully customizable text based on your prompts.

        <Accordion title="Input requirements">
        - Minimum duration: 4 seconds
        - Maximum duration: 2 hours
        - Formats: [FFmpeg supported formats](https://ffmpeg.org/ffmpeg-formats.html)
        - Resolution: 360x360 to 5184x2160 pixels
        - Aspect ratio: Between 1:1 and 1:2.4, or between 2.4:1 and 1:1.
        </Accordion>

        **When to use this method**:
        - Analyze videos longer than 1 hour
        - Process videos asynchronously without blocking your application

        **Do not use this method for**:
        - Videos for which you need immediate results or real-time streaming. Use the [`POST`](/v1.3/api-reference/analyze-videos/sync-analysis) method of the `/analyze` endpoint instead.

        Analyzing videos asynchronously requires three steps:

        1. Create an analysis task using this method. The platform returns a task ID.
        2. Poll the status of the task using the [`GET`](/v1.3/api-reference/analyze-videos/retrieve-analysis-task-status-results) method of the `/analyze/tasks/{task_id}` endpoint. Wait until the status is `ready`.
        3. Retrieve the results from the response when the status is `ready` using the [`GET`](/v1.3/api-reference/analyze-videos/retrieve-analysis-task-status-results) method of the `/analyze/tasks/{task_id}` endpoint.

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        video : VideoContext

        prompt : AnalyzeTextPrompt

        temperature : typing.Optional[AnalyzeTemperature]

        max_tokens : typing.Optional[AnalyzeMaxTokens]

        response_format : typing.Optional[ResponseFormat]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateAnalyzeTaskResponse]
            An analysis task has successfully been created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "analyze/tasks",
            method="POST",
            json={
                "video": convert_and_respect_annotation_metadata(
                    object_=video, annotation=VideoContext, direction="write"
                ),
                "prompt": prompt,
                "temperature": temperature,
                "max_tokens": max_tokens,
                "response_format": convert_and_respect_annotation_metadata(
                    object_=response_format, annotation=ResponseFormat, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAnalyzeTaskResponse,
                    parse_obj_as(
                        type_=CreateAnalyzeTaskResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def retrieve(
        self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AnalyzeTaskResponse]:
        """
        This method retrieves the status and results of an analysis task.

        **Task statuses**:
        - `queued`: The task is waiting to be processed.
        - `pending`: The task is queued and waiting to start.
        - `processing`: The platform is analyzing the video.
        - `ready`: Processing is complete. Results are available in the response.
        - `failed`: The task failed. No results were generated.

        Poll this method until `status` is `ready` or `failed`. When `status` is `ready`, use the results from the response.

        Parameters
        ----------
        task_id : str
            The unique identifier of the analysis task.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AnalyzeTaskResponse]
            Task status and results retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"analyze/tasks/{jsonable_encoder(task_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AnalyzeTaskResponse,
                    parse_obj_as(
                        type_=AnalyzeTaskResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, task_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        This method deletes an analysis task. You can only delete tasks that are not currently being processed.

        Parameters
        ----------
        task_id : str
            The unique identifier of the analyze task.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"analyze/tasks/{jsonable_encoder(task_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AnalyzeMaxTokens,
    AnalyzeTaskError,
    AnalyzeTaskResponse,
    AnalyzeTaskResult,
    AnalyzeTaskResultUsage,
    AnalyzeTaskStatus,
    AnalyzeTaskWebhookInfo,
    AnalyzeTemperature,
    AnalyzeTextPrompt,
    Asset,
    AssetMethod,
    AssetStatus,
    AudioEmbeddingMetadata,
    AudioEmbeddingResult,
    AudioInputRequest,
    AudioInputRequestEmbeddingOptionItem,
    AudioInputRequestEmbeddingScopeItem,
    AudioInputRequestEmbeddingTypeItem,
    AudioSegment,
    AudioSegmentation,
    AudioSegmentationFixed,
    AudioSegmentationStrategy,
    BadRequestErrorBody,
    BaseEmbeddingMetadata,
    BaseSegment,
    BulkCreateEntityResponse,
    BulkCreateEntityResponseEntitiesItem,
    BulkCreateEntityResponseErrorsItem,
    ChunkInfo,
    ChunkInfoStatus,
    CompletedChunk,
    CompletedChunkProofType,
    CreateAnalyzeTaskResponse,
    CreateAssetUploadResponse,
    CreatedAt,
    EmbeddingAudioMetadata,
    EmbeddingAudioMetadataEmbeddingScopesItem,
    EmbeddingData,
    EmbeddingDataEmbeddingOption,
    EmbeddingDataEmbeddingScope,
    EmbeddingImageMetadata,
    EmbeddingMediaMetadata,
    EmbeddingMediaMetadata_Audio,
    EmbeddingMediaMetadata_Image,
    EmbeddingMediaMetadata_MultiInput,
    EmbeddingMediaMetadata_TextImage,
    EmbeddingMediaMetadata_Video,
    EmbeddingMultiInputMetadata,
    EmbeddingResponse,
    EmbeddingSuccessResponse,
    EmbeddingTaskMediaMetadata,
    EmbeddingTaskMediaMetadata_Audio,
    EmbeddingTaskMediaMetadata_Video,
    EmbeddingTaskResponse,
    EmbeddingTaskResponseStatus,
    EmbeddingTextImageMetadata,
    EmbeddingVideoMetadata,
    EmbeddingVideoMetadataEmbeddingScopesItem,
    EndOffsetSec,
    EndTime,
    Entity,
    EntityCollection,
    EntityStatus,
    ErrorResponse,
    ErrorResponseError,
    ExpiresAt,
    FinishReason,
    ForbiddenErrorBody,
    GeneratedTextData,
    GetUploadStatusResponse,
    HlsObject,
    HlsObjectStatus,
    ImageEmbeddingResult,
    ImageInputRequest,
    ImportLog,
    ImportLogFailedFilesItem,
    ImportLogVideoStatus,
    IncompleteUploadSummary,
    IndexModelsItem,
    IndexSchema,
    IndexedAsset,
    IndexedAssetDetailed,
    IndexedAssetDetailedEmbedding,
    IndexedAssetDetailedEmbeddingVideoEmbedding,
    IndexedAssetStatus,
    IndexedAssetSystemMetadata,
    InternalEntity,
    InternalServerErrorBody,
    LimitPerPageSimple,
    ListIncompleteUploadsResponse,
    MediaEmbeddingTask,
    MediaEmbeddingTaskAudioEmbedding,
    MediaEmbeddingTaskVideoEmbedding,
    MediaSource,
    MultiInputMediaSource,
    MultiInputMediaSourceMediaType,
    MultiInputRequest,
    MultipartUploadStatusType,
    NextPageToken,
    NonStreamAnalyzeResponse,
    NotFoundErrorBody,
    One,
    Page,
    PageInfo,
    PresignedUrlChunk,
    PrevPageToken,
    Rank,
    ReportChunkBatchResponse,
    RequestAdditionalPresignedUrLsResponse,
    ResponseFormat,
    ResponseFormatType,
    SearchItem,
    SearchItemClipsItem,
    SearchPool,
    SearchResults,
    SearchResultsPageInfo,
    StartOffsetSec,
    StartTime,
    StreamAnalyzeResponse,
    StreamAnalyzeResponse_StreamEnd,
    StreamAnalyzeResponse_StreamStart,
    StreamAnalyzeResponse_TextGeneration,
    StreamEndResponse,
    StreamEndResponseMetadata,
    StreamStartResponse,
    StreamStartResponseMetadata,
    StreamTextResponse,
    SummarizeChapterResult,
    SummarizeChapterResultChaptersItem,
    SummarizeChapterResultSummarizeType,
    SummarizeHighlightResult,
    SummarizeHighlightResultHighlightsItem,
    SummarizeHighlightResultSummarizeType,
    SummarizeSummaryResult,
    SummarizeSummaryResultSummarizeType,
    TextEmbeddingResult,
    TextImageInputRequest,
    TextInputRequest,
    ThumbnailUrl,
    TokenUsage,
    TotalInnerMatches,
    TotalPage,
    TotalResults,
    TranscriptionData,
    TranscriptionDataItem,
    Two,
    UpdatedAt,
    Url,
    UserMetadata,
    VideoContext,
    VideoContext_AssetId,
    VideoContext_Base64String,
    VideoContext_Url,
    VideoEmbeddingMetadata,
    VideoEmbeddingTask,
    VideoEmbeddingTaskVideoEmbedding,
    VideoIndexingTask,
    VideoIndexingTaskSystemMetadata,
    VideoInputRequest,
    VideoInputRequestEmbeddingOptionItem,
    VideoInputRequestEmbeddingScopeItem,
    VideoInputRequestEmbeddingTypeItem,
    VideoItem,
    VideoItemFailed,
    VideoSegment,
    VideoSegmentation,
    VideoSegmentationDynamic,
    VideoSegmentationDynamicDynamic,
    VideoSegmentationFixed,
    VideoSegmentationFixedFixed,
    VideoSegmentation_Dynamic,
    VideoSegmentation_Fixed,
    VideoVector,
    VideoVectorSystemMetadata,
)
from .errors import (
    BadRequestError,
    ConflictError,
    ForbiddenError,
    InternalServerError,
    NotFoundError,
    TooManyRequestsError,
)
from . import analyze_async, assets, embed, entity_collections, indexes, multipart_upload, search, tasks
from .assets import AssetsCreateRequestMethod, AssetsListRequestAssetTypesItem, AssetsListResponse
from .client import AsyncTwelveLabs, TwelveLabs
from .entity_collections import EntityCollectionsListRequestSortBy, EntityCollectionsListResponse
from .environment import TwelveLabsEnvironment
from .indexes import IndexesCreateRequestModelsItem, IndexesCreateResponse, IndexesListResponse
from .multipart_upload import CreateAssetUploadRequestType
from .search import (
    SearchCreateRequestGroupBy,
    SearchCreateRequestOperator,
    SearchCreateRequestQueryMediaType,
    SearchCreateRequestSearchOptionsItem,
    SearchCreateRequestTranscriptionOptionsItem,
    SearchRetrieveResponse,
    SearchRetrieveResponsePageInfo,
)
from .tasks import (
    TasksCreateResponse,
    TasksListRequestStatusItem,
    TasksListResponse,
    TasksListResponsePageInfo,
    TasksRetrieveResponse,
)
from .version import __version__

__all__ = [
    "AnalyzeMaxTokens",
    "AnalyzeTaskError",
    "AnalyzeTaskResponse",
    "AnalyzeTaskResult",
    "AnalyzeTaskResultUsage",
    "AnalyzeTaskStatus",
    "AnalyzeTaskWebhookInfo",
    "AnalyzeTemperature",
    "AnalyzeTextPrompt",
    "Asset",
    "AssetMethod",
    "AssetStatus",
    "AssetsCreateRequestMethod",
    "AssetsListRequestAssetTypesItem",
    "AssetsListResponse",
    "AsyncTwelveLabs",
    "AudioEmbeddingMetadata",
    "AudioEmbeddingResult",
    "AudioInputRequest",
    "AudioInputRequestEmbeddingOptionItem",
    "AudioInputRequestEmbeddingScopeItem",
    "AudioInputRequestEmbeddingTypeItem",
    "AudioSegment",
    "AudioSegmentation",
    "AudioSegmentationFixed",
    "AudioSegmentationStrategy",
    "BadRequestError",
    "BadRequestErrorBody",
    "BaseEmbeddingMetadata",
    "BaseSegment",
    "BulkCreateEntityResponse",
    "BulkCreateEntityResponseEntitiesItem",
    "BulkCreateEntityResponseErrorsItem",
    "ChunkInfo",
    "ChunkInfoStatus",
    "CompletedChunk",
    "CompletedChunkProofType",
    "ConflictError",
    "CreateAnalyzeTaskResponse",
    "CreateAssetUploadRequestType",
    "CreateAssetUploadResponse",
    "CreatedAt",
    "EmbeddingAudioMetadata",
    "EmbeddingAudioMetadataEmbeddingScopesItem",
    "EmbeddingData",
    "EmbeddingDataEmbeddingOption",
    "EmbeddingDataEmbeddingScope",
    "EmbeddingImageMetadata",
    "EmbeddingMediaMetadata",
    "EmbeddingMediaMetadata_Audio",
    "EmbeddingMediaMetadata_Image",
    "EmbeddingMediaMetadata_MultiInput",
    "EmbeddingMediaMetadata_TextImage",
    "EmbeddingMediaMetadata_Video",
    "EmbeddingMultiInputMetadata",
    "EmbeddingResponse",
    "EmbeddingSuccessResponse",
    "EmbeddingTaskMediaMetadata",
    "EmbeddingTaskMediaMetadata_Audio",
    "EmbeddingTaskMediaMetadata_Video",
    "EmbeddingTaskResponse",
    "EmbeddingTaskResponseStatus",
    "EmbeddingTextImageMetadata",
    "EmbeddingVideoMetadata",
    "EmbeddingVideoMetadataEmbeddingScopesItem",
    "EndOffsetSec",
    "EndTime",
    "Entity",
    "EntityCollection",
    "EntityCollectionsListRequestSortBy",
    "EntityCollectionsListResponse",
    "EntityStatus",
    "ErrorResponse",
    "ErrorResponseError",
    "ExpiresAt",
    "FinishReason",
    "ForbiddenError",
    "ForbiddenErrorBody",
    "GeneratedTextData",
    "GetUploadStatusResponse",
    "HlsObject",
    "HlsObjectStatus",
    "ImageEmbeddingResult",
    "ImageInputRequest",
    "ImportLog",
    "ImportLogFailedFilesItem",
    "ImportLogVideoStatus",
    "IncompleteUploadSummary",
    "IndexModelsItem",
    "IndexSchema",
    "IndexedAsset",
    "IndexedAssetDetailed",
    "IndexedAssetDetailedEmbedding",
    "IndexedAssetDetailedEmbeddingVideoEmbedding",
    "IndexedAssetStatus",
    "IndexedAssetSystemMetadata",
    "IndexesCreateRequestModelsItem",
    "IndexesCreateResponse",
    "IndexesListResponse",
    "InternalEntity",
    "InternalServerError",
    "InternalServerErrorBody",
    "LimitPerPageSimple",
    "ListIncompleteUploadsResponse",
    "MediaEmbeddingTask",
    "MediaEmbeddingTaskAudioEmbedding",
    "MediaEmbeddingTaskVideoEmbedding",
    "MediaSource",
    "MultiInputMediaSource",
    "MultiInputMediaSourceMediaType",
    "MultiInputRequest",
    "MultipartUploadStatusType",
    "NextPageToken",
    "NonStreamAnalyzeResponse",
    "NotFoundError",
    "NotFoundErrorBody",
    "One",
    "Page",
    "PageInfo",
    "PresignedUrlChunk",
    "PrevPageToken",
    "Rank",
    "ReportChunkBatchResponse",
    "RequestAdditionalPresignedUrLsResponse",
    "ResponseFormat",
    "ResponseFormatType",
    "SearchCreateRequestGroupBy",
    "SearchCreateRequestOperator",
    "SearchCreateRequestQueryMediaType",
    "SearchCreateRequestSearchOptionsItem",
    "SearchCreateRequestTranscriptionOptionsItem",
    "SearchItem",
    "SearchItemClipsItem",
    "SearchPool",
    "SearchResults",
    "SearchResultsPageInfo",
    "SearchRetrieveResponse",
    "SearchRetrieveResponsePageInfo",
    "StartOffsetSec",
    "StartTime",
    "StreamAnalyzeResponse",
    "StreamAnalyzeResponse_StreamEnd",
    "StreamAnalyzeResponse_StreamStart",
    "StreamAnalyzeResponse_TextGeneration",
    "StreamEndResponse",
    "StreamEndResponseMetadata",
    "StreamStartResponse",
    "StreamStartResponseMetadata",
    "StreamTextResponse",
    "SummarizeChapterResult",
    "SummarizeChapterResultChaptersItem",
    "SummarizeChapterResultSummarizeType",
    "SummarizeHighlightResult",
    "SummarizeHighlightResultHighlightsItem",
    "SummarizeHighlightResultSummarizeType",
    "SummarizeSummaryResult",
    "SummarizeSummaryResultSummarizeType",
    "TasksCreateResponse",
    "TasksListRequestStatusItem",
    "TasksListResponse",
    "TasksListResponsePageInfo",
    "TasksRetrieveResponse",
    "TextEmbeddingResult",
    "TextImageInputRequest",
    "TextInputRequest",
    "ThumbnailUrl",
    "TokenUsage",
    "TooManyRequestsError",
    "TotalInnerMatches",
    "TotalPage",
    "TotalResults",
    "TranscriptionData",
    "TranscriptionDataItem",
    "TwelveLabs",
    "TwelveLabsEnvironment",
    "Two",
    "UpdatedAt",
    "Url",
    "UserMetadata",
    "VideoContext",
    "VideoContext_AssetId",
    "VideoContext_Base64String",
    "VideoContext_Url",
    "VideoEmbeddingMetadata",
    "VideoEmbeddingTask",
    "VideoEmbeddingTaskVideoEmbedding",
    "VideoIndexingTask",
    "VideoIndexingTaskSystemMetadata",
    "VideoInputRequest",
    "VideoInputRequestEmbeddingOptionItem",
    "VideoInputRequestEmbeddingScopeItem",
    "VideoInputRequestEmbeddingTypeItem",
    "VideoItem",
    "VideoItemFailed",
    "VideoSegment",
    "VideoSegmentation",
    "VideoSegmentationDynamic",
    "VideoSegmentationDynamicDynamic",
    "VideoSegmentationFixed",
    "VideoSegmentationFixedFixed",
    "VideoSegmentation_Dynamic",
    "VideoSegmentation_Fixed",
    "VideoVector",
    "VideoVectorSystemMetadata",
    "__version__",
    "analyze_async",
    "assets",
    "embed",
    "entity_collections",
    "indexes",
    "multipart_upload",
    "search",
    "tasks",
]



# This file was auto-generated by Fern from our API Definition.

import typing

CreateEmbeddingsRequestInputType = typing.Union[
    typing.Literal["text", "image", "text_image", "audio", "video", "multi_input"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    EntitiesCreateBulkRequestEntitiesItem,
    EntitiesListRequestSortBy,
    EntitiesListRequestStatus,
    EntitiesListResponse,
)

__all__ = [
    "EntitiesCreateBulkRequestEntitiesItem",
    "EntitiesListRequestSortBy",
    "EntitiesListRequestStatus",
    "EntitiesListResponse",
]
